@@ -1,6 +1,6 @@
 package Combinator;
 
-use 5.010;
+use 5.006;
 use strict;
 use warnings;
 
@@ -10,11 +10,12 @@ Combinator - Intuitively write async program serially, parallel, or circularly
 
 =head1 VERSION
 
-Version 0.4.3
+Version 0.01
 
 =cut
 
-our $VERSION = '0.004003';
+our $VERSION = '0.01';
+
 
 =head1 SYNOPSIS
 
@@ -26,17 +27,17 @@ The following is the basic form for serializing a sequence of async code blocks:
     my $cv = AE::cv;
     {{com
         print "sleep 1 second\n";
-        my $t = AE::timer 1, 0, {{next}};
+        my $t; $t = AE::timer 1, 0, {{next}};
       --ser
         undef $t;
-        my $t = AE::timer 0.5, 0, {{next}};
+        my $t; $t = AE::timer 0.5, 0, {{next}};
         print "sleep 0.5 second\n"; # this line will be executed before the next block
       --ser
         undef $t;
         print "wait for 3 timers at the same time\n";
-        my $t1 = AE::timer 1, 0, {{next}};
-        my $t2 = AE::timer 2, 0, {{next}};
-        my $t3 = AE::timer 1.5, 0, {{next}};
+        my $t1; $t1 = AE::timer 1, 0, {{next}};
+        my $t2; $t2 = AE::timer 2, 0, {{next}};
+        my $t3; $t3 = AE::timer 1.5, 0, {{next}};
       --ser
         undef $t1; undef $t2; undef $t3;
         # after the max time interval of them (2 seconds)
@@ -47,7 +48,7 @@ The following is the basic form for serializing a sequence of async code blocks:
     }}com
     $cv->recv;
 
-The following block will wait for previous block's end and all the {{next}}s in the
+The following block will wait for previous block's end and all the {{next}} in the
 previous block been called.
 
 And also, it could be nested {{com..}}com blocks in the code block.
@@ -102,7 +103,7 @@ and optionally use 'return' to stop the {{com..}}com block.
     $cv->recv;
 
 And also, the following block will get all the arguments when {{next}} is called.
-This is useful when integrating with other callback based module.
+This is useful when integrate with other callback based module.
 
     use Combinator;
     use AE;
@@ -146,19 +147,16 @@ You'll get all the args concatenated together.
     {{com
         {{next}}->(0);
         {{com
-            my $t = AE::timer 1, 0, {{next}};
+            my $t; $t = AE::timer 1, 0, {{next}};
           --ser
-            undef $t;
             {{next}}->(1);
         --com
-            my $t = AE::timer .6, 0, {{next}};
+            my $t; $t = AE::timer .6, 0, {{next}};
           --ser
-            undef $t;
             {{next}}->(2);
         --com
-            my $t = AE::timer .3, 0, {{next}};
+            my $t; $t = AE::timer .3, 0, {{next}};
           --ser
-            undef $t;
             {{next}}->(3);
         }}com
         {{next}}->(4);
@@ -219,52 +217,6 @@ or use --cir instead of --com if it's not the first block.
 
     AE::cv->recv;
 
-If you need finer controlled {{next}}, use {{nex .. }}nex block to
-replace {{next}}.
-
-    use Combinator;
-    use AE;
-    use AnyEvent::HTTP;
-
-    {{com
-        my($a_res, $b_res);
-        http_get 'http://site.a/', {{nex $a_res = $_[1] }}nex;
-        http_get 'http://site.b/', {{nex $b_res = $_[1] }}nex;
-      --ser
-        print "Completed!\n";
-        print "SiteA = $a_res\n";
-        print "SiteB = $b_res\n";
-    }}com
-
-    AE::cv->recv;
-
-Though without {{nex .. }}nex block, you can still write:
-
-    use Combinator;
-    use AE;
-    use AnyEvent::HTTP;
-
-    {{com
-        my($a_res, $b_res);
-        {{com
-            http_get 'http://site.a/', {{next}};
-          --ser
-            $a_res = $_[1];
-        --com
-            http_get 'http://site.b/', {{next}};
-          --ser
-            $b_res = $_[1];
-        }}com
-      --ser
-        print "Completed!\n";
-        print "SiteA = $a_res\n";
-        print "SiteB = $b_res\n";
-    }}com
-
-    AE::cv->recv;
-
-It's up to you to choose which one to use.
-
 =head1 WHEN YOU SHOULD USE THIS MODULE
 
 =head2 When you are tired of writing layered closures
@@ -418,15 +370,13 @@ Set to 1 if you want to see the generated code.
 
 =head2 cir_begin => qr/\{\{cir\b/
 
-=head2 nex_begin => qr/\{\{nex\b/
-
 =head2 ser => qr/--ser\b/
 
 =head2 par => qr/--com\b/
 
 =head2 cir_par => qr/--cir\b/
 
-=head2 end => qr/\}\}(?:com|cir|nex)\b/
+=head2 end => qr/\}\}com\b/
 
 =head2 next => qr/\{\{next\}\}/
 
@@ -505,6 +455,15 @@ Then "B" will come after "A";
 
 Look up the file eg/demo_all.pl
 
+=head1 EXPORT
+
+A list of functions that can be exported.  You can delete this section
+if you don't export anything, such as for a purely object-oriented module.
+
+=head1 SUBROUTINES/METHODS
+
+=head2 function1
+
 =cut
 
 
@@ -577,7 +536,6 @@ my $par_pat;
 my $cir_par_pat;
 my $com_pat;
 my $token_pat;
-my $nex_begin_pat;
 my $line_shift;
 
 our $cv1 = [];
@@ -588,20 +546,18 @@ sub import {
         verbose => 0,
         begin => qr/\{\{com\b/,
         cir_begin => qr/\{\{cir\b/,
-        nex_begin => qr/\{\{nex\b/,
         ser => qr/--ser\b/,
         par => qr/--com\b/,
         cir_par => qr/--cir\b/,
-        end => qr/\}\}(?:com|cir|nex)\b/,
+        end => qr/\}\}com\b/,
         next => qr/\{\{next\}\}/,
         @_
     );
-    $begin_pat = qr/$opt{begin}|$opt{cir_begin}|(?:$opt{nex_begin})/;
+    $begin_pat = qr/$opt{begin}|$opt{cir_begin}/;
     $end_pat = $opt{end};
     $ser_pat = $opt{ser};
     $par_pat = $opt{par};
     $cir_begin_pat = $opt{cir_begin};
-    $nex_begin_pat = $opt{nex_begin};
     $cir_par_pat = $opt{cir_par};
     $com_pat = qr/($begin_pat((?:(?-2)|(?!$begin_pat).)*?)$end_pat)/s;
     $token_pat = qr/$com_pat|(?!$begin_pat)./s;
@@ -620,11 +576,9 @@ sub att_sub {
 sub cv_end { # (cv, args)
     --$_[0][0];
     push @{$_[0][2]//=[]}, @{$_[1]} if $_[1];
-    if( !$_[0][0] ) {
-        if( $_[0][1] ) {
-            delete($_[0][1])->(@{$_[0][2]});
-        }
-        undef $_[0][2];
+    if( !$_[0][0] && $_[0][1] ) {
+        my $cb = delete $_[0][1];
+        $cb->(@{$_[0][2]});
     }
 }
 sub cv_cb { # (cv, cb)
@@ -633,7 +587,6 @@ sub cv_cb { # (cv, cb)
     }
     else {
         $_[1](@{$_[0][2]});
-        undef $_[0][2];
     }
 }
 
@@ -646,35 +599,19 @@ sub ser {
     unshift @_, $depth;
     my $next = &ser;
     replace_code($depth, $code);
-    $code =~ s/$opt{next}/(do{my\$t=\$Combinator::cv1;++\$t->[0];sub{if(\$t){Combinator::cv_end(\$t,\\\@_);undef\$t}else{my(undef,\$f,\$l)=caller;warn"next should be invoked only once at \$f line \$l.\\n"}}})/g;
+    $code =~ s/$opt{next}/(do{my\$t=\$Combinator::cv1;++\$t->[0];sub{Combinator::cv_end(\$t,\\\@_)}})/g;
     my $out = "local\$Combinator::guard=Guard::guard{Combinator::cv_end(\$Combinator::cv0,\\\@_)};local\$Combinator::cv1=[1];$code;--\$Combinator::cv1->[0];Combinator::cv_cb(\$Combinator::cv1,Combinator::att_sub(\$Combinator::head,\$Combinator::cv0,sub{local\$Combinator::head=shift;local\$Combinator::cv0=shift;$next}));\$Combinator::guard->cancel";
     return $out;
 }
 
-sub com { # depth, code, head
-    my($depth, $code, $head) = @_;
+sub com { # depth, code, cir
+    my($depth, $code, $cir) = @_;
     my @ser;
     $code .= "\n" if( substr($code, -1) eq "\n" );
     push @ser, $1 while( $code =~ m/(?:^|$ser_pat)($token_pat*?)(?=$ser_pat|$)/gs );
-
-    if( @ser == 1 && $head !~ $cir_par_pat && $head !~ $cir_begin_pat && $head !~ $nex_begin_pat ) {
-        replace_code($depth, @ser);
-        return "{$ser[0]}";
-    }
-
-    my $delayed = $head =~ $nex_begin_pat;
-
-    my $out = (
-            $delayed ?
-                "(do{++\$Combinator::cv1->[0];Combinator::att_sub(do{\\(my\$t=1)},\$Combinator::cv1,sub{if(!\${\$_[0]}){my(undef,\$f,\$l)=caller;warn\"nex should be invoked only once at \$f line \$l.\\n\";return}--\${\$_[0]};shift;local\$Combinator::cv0=shift;" :
-                "{&{sub{local\$Combinator::cv0=\$Combinator::cv1;++\$Combinator::cv0->[0];"
-        )."local\$Combinator::head=[1,Devel::Caller::caller_cv(0)];" .
-        ser($depth+1, @ser, $head =~ /^(?:$cir_par_pat|$cir_begin_pat)$/ ? "--\$Combinator::cv0->[0];\$Combinator::cv1=\$Combinator::cv0;Combinator::cv_end(\$Combinator::head,\\\@_)" : "Combinator::cv_end(\$Combinator::cv0,\\\@_)") .
-        (
-            $delayed ?
-                "})})" :
-                "}}}"
-        );
+    my $out = "{&{sub{local\$Combinator::head=[1,Devel::Caller::caller_cv(0)];local\$Combinator::cv0=\$Combinator::cv1;++\$Combinator::cv0->[0];" .
+        ser($depth+1, @ser, $cir ? "--\$Combinator::cv0->[0];\$Combinator::cv1=\$Combinator::cv0;Combinator::cv_end(\$Combinator::head,\\\@_)" : "Combinator::cv_end(\$Combinator::cv0,\\\@_)") .
+        "}}}";
     return $out;
 }
 
@@ -685,7 +622,7 @@ sub replace_code {
         my $out = '';
         while( $code =~ /($begin_pat|$par_pat|$cir_par_pat)($token_pat*?)(?=($par_pat|$cir_par_pat|$end_pat))/g ) {
             my $fragment = $2;
-            $out .= com($depth, $fragment, $1);
+            $out .= com($depth, $fragment, $1 =~ /^(?:$cir_par_pat|$cir_begin_pat)$/);
         }
         $out;
     }ge;
